// WinBorder.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

public class WinBorder extends Border {
    public static final int RAISED_BUTTON  = 0;
    public static final int LOWERED_BUTTON = 1;
    public static final int RAISED_SCROLL_BUTTON  = 2;
    public static final int LOWERED_SCROLL_BUTTON = 3;
    public static final int LOWERED_BEZEL = 4;
    public static final int SHALLOW_LOWERED_BEZEL = 5;
    public static final int SHALLOW_RAISED_BEZEL = 6;
    public static final int GROOVED_BEZEL = 7;

    private static WinBorder raisedButton;
    private static WinBorder loweredButton;
    private static WinBorder raisedScrollButton;
    private static WinBorder loweredScrollButton;
    private static WinBorder loweredBezel;
    private static WinBorder shallowLoweredBezel;
    private static WinBorder shallowRaisedBezel;
    private static Border groovedBezel;

    final static String TYPE_KEY = "type";

    int type;
    float factor = (float)1.0;

    public static Border raisedButton() {
        if (raisedButton == null) {
            raisedButton = new WinBorder(RAISED_BUTTON);
        }

        return raisedButton;
    }

    public static Border loweredButton() {
        if (loweredButton == null) {
            loweredButton = new WinBorder(LOWERED_BUTTON);
        }

        return loweredButton;
    }

    public static Border raisedScrollButton() {
        if (raisedScrollButton == null) {
            raisedScrollButton = new WinBorder(RAISED_SCROLL_BUTTON);
        }

        return raisedScrollButton;
    }

    public static Border loweredScrollButton() {
        if (loweredScrollButton == null) {
            loweredScrollButton = new WinBorder(LOWERED_SCROLL_BUTTON);
        }

        return loweredScrollButton;
    }

    public static Border loweredBezel() {
        if (loweredBezel == null) {
            loweredBezel = new WinBorder(LOWERED_BEZEL);
        }

        return loweredBezel;
    }

    public static Border shallowLoweredBezel() {
        if (shallowLoweredBezel == null) {
            shallowLoweredBezel = new WinBorder(SHALLOW_LOWERED_BEZEL);
        }

        return shallowLoweredBezel;
    }

    public static Border shallowRaisedBezel() {
        if (shallowRaisedBezel == null) {
            shallowRaisedBezel = new WinBorder(SHALLOW_RAISED_BEZEL);
        }

        return shallowRaisedBezel;
    }

    public static Border groovedBezel() {
        if (groovedBezel == null) {
            groovedBezel = new BezelBorder(BezelBorder.GROOVED, 
                                                WinColor.control, 
                                                WinColor.controlLtHighlight, 
                                                WinColor.controlShadow);
        }

        return groovedBezel;
    }

    public WinBorder(int type) {
        super();
        this.type = type;
    }

    public WinBorder() {
        super();
    }

    public void setFactor(float factor) {
        this.factor = factor;
    }

    public float factor() {
        return factor;
    }

    public int leftMargin() {
        return 2;
    }

    public int rightMargin() {
        return 2;
    }

    public int topMargin() {
        return 2;
    }

    public int bottomMargin() {
        return 2;
    }

    public int type() {
        return type;
    }

    Color factoredColor(Color sColor) {
        Color dColor = WinColor.control;
        int dRed = dColor.red(), dGreen = dColor.green(), dBlue = dColor.blue();
        int sRed = sColor.red(), sGreen = sColor.green(), sBlue = sColor.blue();
        int red = (int)(factor * (float)(sRed - dRed)) + dRed;
        int blue = (int)(factor * (float)(sBlue - dBlue)) + dBlue;
        int green = (int)(factor * (float)(sGreen - dGreen)) + dGreen;

//        System.out.println("factor: " + factor + " source color: " + sColor + " dest color: " + dColor + " new color: " + new Color(red, green, blue));
        return new Color(red, green, blue);
    }

    public void drawInRect(Graphics g, int x, int y, int w, int h) {
        Color outerNW, outerSE, innerNW, innerSE;

        if (w == 0 || h == 0)
            return;

        switch (type) {
        case RAISED_BUTTON:
            outerNW = WinColor.controlLtHighlight;
            outerSE = WinColor.controlDkShadow;
            innerNW = WinColor.controlHighlight;
            innerSE = WinColor.controlShadow;
            break;
        case LOWERED_BUTTON:
            outerNW = WinColor.controlDkShadow;
            outerSE = WinColor.controlLtHighlight;
            innerNW = WinColor.controlShadow;
            innerSE = WinColor.controlHighlight;
            break;
        case LOWERED_BEZEL:
            outerNW = WinColor.controlShadow;
            outerSE = WinColor.controlLtHighlight;
            innerNW = WinColor.controlDkShadow;
            innerSE = WinColor.controlHighlight;
            break;
        case RAISED_SCROLL_BUTTON:
            outerNW = WinColor.controlHighlight;
            outerSE = WinColor.controlDkShadow;
            innerNW = WinColor.controlLtHighlight;
            innerSE = WinColor.controlShadow;
            break;
        case LOWERED_SCROLL_BUTTON:
            outerNW = WinColor.controlShadow;
            outerSE = WinColor.controlShadow;
            innerNW = WinColor.control;
            innerSE = WinColor.control;
            break;

        case SHALLOW_LOWERED_BEZEL:
            outerNW = WinColor.controlShadow;
            outerSE = WinColor.controlLtHighlight;
            innerNW = null;
            innerSE = null;
            break;

        case SHALLOW_RAISED_BEZEL:
            outerNW = WinColor.controlLtHighlight;
            outerSE = WinColor.controlShadow;
            innerNW = null;
            innerSE = null;
            break;

        case GROOVED_BEZEL:
            groovedBezel().drawInRect(g, x, y, w, h);
            return;

        default:
            throw new RuntimeException("Unknown type: " + type);
        }

        if (outerNW != null) {
            g.setColor(factoredColor(outerNW));
            g.fillRect(x, y, w - 1, 1);
            g.fillRect(x, y + 1, 1, h - 1);
        }

        if (outerSE != null) {
            g.setColor(factoredColor(outerSE));
            g.fillRect(x, y + h - 1, w, 1);
            g.fillRect(x + w - 1, y, 1, h);
        }

        x++;
        y++;
        w -= 2;
        h -= 2;

        if (innerNW != null) {
            g.setColor(factoredColor(innerNW));
            g.fillRect(x, y, w - 1, 1);
            g.fillRect(x, y + 1, 1, h - 1);
        }

        if (innerSE != null) {
            g.setColor(factoredColor(innerSE));
            g.fillRect(x, y + h - 1, w, 1);
            g.fillRect(x + w - 1, y, 1, h);
        }
    }

    public void describeClassInfo(ClassInfo info) {
        super.describeClassInfo(info);
        info.addField(TYPE_KEY, INT_TYPE);
    }

    public void encode(Encoder encoder) throws CodingException {
        super.encode(encoder);
        encoder.encodeInt(TYPE_KEY, type);
    }

    public void decode(Decoder decoder) throws CodingException {
        super.decode(decoder);
        type = decoder.decodeInt(TYPE_KEY);
    }


}
