// WinButton.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

// XXX Need override some methods to handle enable/disable, textHighlightText,
// etc.

public class WinButton extends Button {
    public static final String MOUSE_ENTERED_COMMAND = "mouseEntered";
    public static final String MOUSE_EXITED_COMMAND = "mouseExited";

    Target rollOverTarget;
    String rollOverMessage;
    boolean isDefault;
    Border originalLoweredBorder;
    boolean isFocusedButton;
    boolean isSelectable = true;
    Image disabledImage;
    Image altDisabledImage;
    View nextSelectView, prevSelectView;

    public static WinButton createWinCheckButton(int x, int y, int width,
        int height) {
        WinButton  checkButton;

        checkButton = new WinButton(x, y, width, height);

        checkButton.setType(Button.TOGGLE_TYPE);
        checkButton.setTransparent(true);

        checkButton.setRaisedBorder(null);
        checkButton.setLoweredBorder(null);

        checkButton.setImage(WinCheckImage.uncheckedImage());
        checkButton.setAltImage(WinCheckImage.checkedImage());
        checkButton.setDisabledImage(WinCheckImage.disabledUncheckedImage());
        checkButton.setAltDisabledImage(WinCheckImage.disabledCheckedImage());
        checkButton.setImagePosition(Button.IMAGE_ON_LEFT);

        return checkButton;
    }

    public WinButton(int x, int y, int w, int h) {
        super(x, y, w, h);

        setRaisedBorder(WinBorder.raisedButton());
        setLoweredBorder(WinBorder.loweredButton());
        setRaisedColor(WinColor.control);
        setLoweredColor(WinColor.control);
        setTitleColor(WinColor.controlText);
        setDisabledTitleColor(WinColor.textInactiveText);
        setFont(WinFont.defaultFont());
    }

    public WinButton() {
        this(0, 0, 0, 0);
    }

    public void setRollOverMessage(String message) {
        rollOverMessage = message;
    }

    public String rollOverMessage() {
        return rollOverMessage;
    }

    public void setRollOverTarget(Target rollOverTarget) {
        this.rollOverTarget = rollOverTarget;
    }

    public Target rollOverTarget() {
        return rollOverTarget;
    }

    public void mouseEntered(MouseEvent event) {
        if (rollOverTarget != null)
            rollOverTarget.performCommand(MOUSE_ENTERED_COMMAND, this);
    }

    public void mouseExited(MouseEvent event) {
        if (rollOverTarget != null)
            rollOverTarget.performCommand(MOUSE_EXITED_COMMAND, this);
    }

    public void drawView(Graphics g) {
        Image _normalImage = null;
        Image _normalAltImage = null;

        if(!isEnabled()) {
            _normalImage = image();
            _normalAltImage = altImage();
            if(disabledImage != null)
                setImage(disabledImage);
            if(altDisabledImage != null)
                setAltImage(altDisabledImage);
        }
        super.drawView(g);
        drawSelectionRect(g);
        if(!isEnabled()) {
            setImage(_normalImage);
            setAltImage(_normalAltImage);
        }

    }

    public void drawSelectionRect(Graphics g) {
        Size            imageAreaSize, stringSize;
        int             x, charHeight, justification;
        boolean         center;
        Rect interiorRect = localBounds();
        Font font;
        FontMetrics fm;
        int w2;

        if(!isFocusedButton) {
            return;
        }

        // Normal buttons are easy to draw
        if(type() != TOGGLE_TYPE) {
            g.setColor(WinColor.controlDkShadow);
            Rect r = localBounds();
            r.growBy(-4, -4);
            drawDashedRect(g, r);
            return;
        }

        // TOGGLE_TYPE buttons are a little more complicated

        // This was hijacked out of netscape.application.Button
        // to determine the Rect of the title string
        imageAreaSize = imageAreaSize();
        font = g.font();
        if(font == null)
            w2 = -1;
        else {
            fm = font.fontMetrics();
            w2 = fm.stringWidth(title());
        }

        if (imagePosition() == IMAGE_ON_LEFT) {
            if (title() == null || title().length() == 0) {
                x = interiorRect.x + 1 +
                    (interiorRect.width - imageAreaSize.width - 2) / 2;
            } else {
                x = interiorRect.x + 1;
            }

            /* put some distance between the image and the text */
            if (imageAreaSize.width > 0) {
                interiorRect.moveBy(imageAreaSize.width + 3, 0);
                interiorRect.sizeBy(-(imageAreaSize.width + 4), 0);
                justification = Graphics.LEFT_JUSTIFIED;
            } else {
                interiorRect.moveBy(1, 0);
                interiorRect.sizeBy(-2, 0);
                justification = Graphics.CENTERED;
            }

            /// Ok now tweak the rect to make the selection rect be
            /// the right size
            g.setColor(WinColor.controlDkShadow);
            interiorRect.x = interiorRect.x - 2;
            interiorRect.y = interiorRect.y + 2;
            interiorRect.height = interiorRect.height - 2;
            if(w2 > 0)
                interiorRect.width = w2 + 4;
            else
                interiorRect.width = interiorRect.width - 2;
            drawDashedRect(g, interiorRect);
        }

    }

    /* Controls if the component wants to participate in the selection machinery. */
    public boolean canBecomeSelectedView() {
        return isSelectable && isEnabled();
    }

    public void setDisabledImage(Image i) { disabledImage = i; }
    public void setAltDisabledImage(Image i) { altDisabledImage = i; }

    /*If this is false, then the default selection machinery will ignore this button.
     * this is true by default
     */
    public void setSelectable(boolean value) {
        isSelectable = value;
    }

    public void willBecomeSelected() {
        setFocusedButton(true);
    }

    public void willBecomeUnselected() {
        setFocusedButton(false);
    }

    /*Draws the black outline around the button and used the flat depress border.
     * You are responsible for sending the command to this button at the appropriate
     * time.
     */
    public void setDefault(boolean value) {
        isDefault = value;
        if(isDefault) {
           setRaisedBorder(new CompoundBorder(LineBorder.blackLine(),
                                        raisedBorder()));
           originalLoweredBorder = loweredBorder();
           setLoweredBorder(new CompoundBorder(LineBorder.blackLine(),
                                        new LineBorder(WinColor.controlShadow)));
        } else {
            setRaisedBorder(((CompoundBorder)raisedBorder()).innerBorder());
            setLoweredBorder(originalLoweredBorder);
            originalLoweredBorder = null;
        }
    }

    public boolean isDefaultButton() { return isDefault; }

    /* adds binding for space key */
    public void setFocusedButton(boolean value) {
        isFocusedButton = value;
        setDirty(true);
        if(value) {
           setCommandForKey("click", null, ' ',
                                    KeyEvent.NO_MODIFIERS_MASK,
                                    View.WHEN_IN_MAIN_WINDOW);

        } else {
           setCommandForKey(null, null, ' ',
                                    KeyEvent.NO_MODIFIERS_MASK,
                                    View.WHEN_IN_MAIN_WINDOW);
        }
    }

    public boolean isFocusedButton() {
        return isFocusedButton;
    }


    public void drawDashedRect(Graphics g, Rect r) {
        int _x,_y, bx, by, bw, bh;

        bx = r.x;
        by = r.y;
        bw = r.width;
        bh = r.height;

        // we always draw on odd boundries, the lines look better,
        if((bw % 2) == 0)
           bw--;

        if((bh % 2) == 0)
           bh--;

        // draw upper and lower horizontal dashes
        for (_x = bx; _x < (bx + bw); _x += 2) {
            g.drawLine(_x, by, _x, by);
            g.drawLine(_x, by + bh - 1, _x, by + bh - 1);
        }

        // draw left and right vertical dashes
        for (_y = by; _y < (by + bh ); _y += 2) {
            g.drawLine(bx, _y, bx, _y);
            g.drawLine(bx + bw - 1, _y, bx + bw - 1, _y);
        }

    }

    public void setNextView(View view) { nextSelectView = view; invalidateKeyboardSelectionOrder(); }
    public View nextSelectableView() {
        if(nextSelectView != null)
            return nextSelectView;
        return super.nextSelectableView();
    }

    public void setPrevView(View view) { prevSelectView = view; }
        public View previousSelectableView() {
        if(prevSelectView != null)
            return prevSelectView;
        return super.previousSelectableView();
    }

    public void decode(Decoder decoder) throws CodingException {
        super.decode(decoder);
        setRaisedColor(WinColor.control);
        setLoweredColor(WinColor.control);
        setTitleColor(WinColor.controlText);
        setDisabledTitleColor(WinColor.textInactiveText);
        setFont(WinFont.defaultFont());
    }
}
