// WinButton.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

public class WinProgressBar extends View {

    Border outerBorder;
    int totalCount, currentCount;

    int tickY, tickH, tickCount, currentTick;

    boolean done;

    static final int tickInset = 3;
    static final int tickOffset = 3;
    static final int tickThickness = 10;

    public WinProgressBar() {
        this(0, 0, 0, 0);
    }

    public WinProgressBar(int x, int y, int w, int h) {
        super(x, y, w, h);

        outerBorder = WinBorder.shallowLoweredBezel();
        done = false;

        setTotalCount(0);
        setCurrentCount(0);

        setBuffered(true);
    }

    protected void calculateTicks() {
        tickY = tickInset;
        tickH = height() - 2*tickInset;

        tickCount = ((width() - 2*tickOffset)/tickOffset) + 1;
    }

    protected void calculateCurrentTick() {
        if (totalCount == 0) {
            if (done) {
                currentTick = tickCount - 1;
            } else {
                currentTick = 0;
            }

        } else {
            currentTick = (currentCount*(tickCount-1)/totalCount);
        }
    }

    public void setTotalCount(int totalCount) {
        this.totalCount = totalCount;
        calculateTicks();
    }

    public void setCurrentCount(int currentCount) {
        this.currentCount = currentCount;
        calculateCurrentTick();
    }

    public void setDone(boolean done) {
        this.done = done;
        calculateCurrentTick();
        setDirty(true);
    }

    public int currentX() {
        return bounds().x + tickOffset*(currentTick + 1);
    }

    public void drawView(Graphics g) {
        Rect r;
        int i, x, currentX;
        boolean hit = false;

        r = new Rect(0, 0, width(), height());
        outerBorder.drawInRect(g, r);

        g.setColor(WinColor.activeCaption);

        currentX = tickOffset*(currentTick + 1);

        for (i = 0; i < tickCount; i++) {
            x = (tickOffset*(i+1)) + ((tickThickness - 1) * i);

            if (!hit && x >= currentX) {
                g.setColor(WinColor.text);
                hit = true;
            } 
            if ((x >= g.clipRect().x) && (x <= (g.clipRect().x + g.clipRect().width))) {
                g.fillRect(x, tickY, tickThickness, tickH);
            }
        }
    }
}





